"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = thunkify;
/**
 * Copyright 2018 Shape Security, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License")
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

function thunkify(reducer) {
  return {
    reduceArrayAssignmentTarget: function reduceArrayAssignmentTarget(node, _ref) {
      var elements = _ref.elements,
          rest = _ref.rest;

      return reducer.reduceArrayAssignmentTarget(node, { elements: elements.map(function (n) {
          return n == null ? null : n();
        }), rest: rest == null ? null : rest() });
    },
    reduceArrayBinding: function reduceArrayBinding(node, _ref2) {
      var elements = _ref2.elements,
          rest = _ref2.rest;

      return reducer.reduceArrayBinding(node, { elements: elements.map(function (n) {
          return n == null ? null : n();
        }), rest: rest == null ? null : rest() });
    },
    reduceArrayExpression: function reduceArrayExpression(node, _ref3) {
      var elements = _ref3.elements;

      return reducer.reduceArrayExpression(node, { elements: elements.map(function (n) {
          return n == null ? null : n();
        }) });
    },
    reduceArrowExpression: function reduceArrowExpression(node, _ref4) {
      var params = _ref4.params,
          body = _ref4.body;

      return reducer.reduceArrowExpression(node, { params: params(), body: body() });
    },
    reduceAssignmentExpression: function reduceAssignmentExpression(node, _ref5) {
      var binding = _ref5.binding,
          expression = _ref5.expression;

      return reducer.reduceAssignmentExpression(node, { binding: binding(), expression: expression() });
    },
    reduceAssignmentTargetIdentifier: function reduceAssignmentTargetIdentifier(node) {
      return reducer.reduceAssignmentTargetIdentifier(node);
    },
    reduceAssignmentTargetPropertyIdentifier: function reduceAssignmentTargetPropertyIdentifier(node, _ref6) {
      var binding = _ref6.binding,
          init = _ref6.init;

      return reducer.reduceAssignmentTargetPropertyIdentifier(node, { binding: binding(), init: init == null ? null : init() });
    },
    reduceAssignmentTargetPropertyProperty: function reduceAssignmentTargetPropertyProperty(node, _ref7) {
      var name = _ref7.name,
          binding = _ref7.binding;

      return reducer.reduceAssignmentTargetPropertyProperty(node, { name: name(), binding: binding() });
    },
    reduceAssignmentTargetWithDefault: function reduceAssignmentTargetWithDefault(node, _ref8) {
      var binding = _ref8.binding,
          init = _ref8.init;

      return reducer.reduceAssignmentTargetWithDefault(node, { binding: binding(), init: init() });
    },
    reduceBinaryExpression: function reduceBinaryExpression(node, _ref9) {
      var left = _ref9.left,
          right = _ref9.right;

      return reducer.reduceBinaryExpression(node, { left: left(), right: right() });
    },
    reduceBindingIdentifier: function reduceBindingIdentifier(node) {
      return reducer.reduceBindingIdentifier(node);
    },
    reduceBindingPropertyIdentifier: function reduceBindingPropertyIdentifier(node, _ref10) {
      var binding = _ref10.binding,
          init = _ref10.init;

      return reducer.reduceBindingPropertyIdentifier(node, { binding: binding(), init: init == null ? null : init() });
    },
    reduceBindingPropertyProperty: function reduceBindingPropertyProperty(node, _ref11) {
      var name = _ref11.name,
          binding = _ref11.binding;

      return reducer.reduceBindingPropertyProperty(node, { name: name(), binding: binding() });
    },
    reduceBindingWithDefault: function reduceBindingWithDefault(node, _ref12) {
      var binding = _ref12.binding,
          init = _ref12.init;

      return reducer.reduceBindingWithDefault(node, { binding: binding(), init: init() });
    },
    reduceBlock: function reduceBlock(node, _ref13) {
      var statements = _ref13.statements;

      return reducer.reduceBlock(node, { statements: statements.map(function (n) {
          return n();
        }) });
    },
    reduceBlockStatement: function reduceBlockStatement(node, _ref14) {
      var block = _ref14.block;

      return reducer.reduceBlockStatement(node, { block: block() });
    },
    reduceBreakStatement: function reduceBreakStatement(node) {
      return reducer.reduceBreakStatement(node);
    },
    reduceCallExpression: function reduceCallExpression(node, _ref15) {
      var callee = _ref15.callee,
          _arguments = _ref15.arguments;

      return reducer.reduceCallExpression(node, { callee: callee(), _arguments: _arguments.map(function (n) {
          return n();
        }) });
    },
    reduceCatchClause: function reduceCatchClause(node, _ref16) {
      var binding = _ref16.binding,
          body = _ref16.body;

      return reducer.reduceCatchClause(node, { binding: binding(), body: body() });
    },
    reduceClassDeclaration: function reduceClassDeclaration(node, _ref17) {
      var name = _ref17.name,
          _super = _ref17.super,
          elements = _ref17.elements;

      return reducer.reduceClassDeclaration(node, { name: name(), _super: _super == null ? null : _super(), elements: elements.map(function (n) {
          return n();
        }) });
    },
    reduceClassElement: function reduceClassElement(node, _ref18) {
      var method = _ref18.method;

      return reducer.reduceClassElement(node, { method: method() });
    },
    reduceClassExpression: function reduceClassExpression(node, _ref19) {
      var name = _ref19.name,
          _super = _ref19.super,
          elements = _ref19.elements;

      return reducer.reduceClassExpression(node, { name: name == null ? null : name(), _super: _super == null ? null : _super(), elements: elements.map(function (n) {
          return n();
        }) });
    },
    reduceCompoundAssignmentExpression: function reduceCompoundAssignmentExpression(node, _ref20) {
      var binding = _ref20.binding,
          expression = _ref20.expression;

      return reducer.reduceCompoundAssignmentExpression(node, { binding: binding(), expression: expression() });
    },
    reduceComputedMemberAssignmentTarget: function reduceComputedMemberAssignmentTarget(node, _ref21) {
      var object = _ref21.object,
          expression = _ref21.expression;

      return reducer.reduceComputedMemberAssignmentTarget(node, { object: object(), expression: expression() });
    },
    reduceComputedMemberExpression: function reduceComputedMemberExpression(node, _ref22) {
      var object = _ref22.object,
          expression = _ref22.expression;

      return reducer.reduceComputedMemberExpression(node, { object: object(), expression: expression() });
    },
    reduceComputedPropertyName: function reduceComputedPropertyName(node, _ref23) {
      var expression = _ref23.expression;

      return reducer.reduceComputedPropertyName(node, { expression: expression() });
    },
    reduceConditionalExpression: function reduceConditionalExpression(node, _ref24) {
      var test = _ref24.test,
          consequent = _ref24.consequent,
          alternate = _ref24.alternate;

      return reducer.reduceConditionalExpression(node, { test: test(), consequent: consequent(), alternate: alternate() });
    },
    reduceContinueStatement: function reduceContinueStatement(node) {
      return reducer.reduceContinueStatement(node);
    },
    reduceDataProperty: function reduceDataProperty(node, _ref25) {
      var name = _ref25.name,
          expression = _ref25.expression;

      return reducer.reduceDataProperty(node, { name: name(), expression: expression() });
    },
    reduceDebuggerStatement: function reduceDebuggerStatement(node) {
      return reducer.reduceDebuggerStatement(node);
    },
    reduceDirective: function reduceDirective(node) {
      return reducer.reduceDirective(node);
    },
    reduceDoWhileStatement: function reduceDoWhileStatement(node, _ref26) {
      var body = _ref26.body,
          test = _ref26.test;

      return reducer.reduceDoWhileStatement(node, { body: body(), test: test() });
    },
    reduceEmptyStatement: function reduceEmptyStatement(node) {
      return reducer.reduceEmptyStatement(node);
    },
    reduceExport: function reduceExport(node, _ref27) {
      var declaration = _ref27.declaration;

      return reducer.reduceExport(node, { declaration: declaration() });
    },
    reduceExportAllFrom: function reduceExportAllFrom(node) {
      return reducer.reduceExportAllFrom(node);
    },
    reduceExportDefault: function reduceExportDefault(node, _ref28) {
      var body = _ref28.body;

      return reducer.reduceExportDefault(node, { body: body() });
    },
    reduceExportFrom: function reduceExportFrom(node, _ref29) {
      var namedExports = _ref29.namedExports;

      return reducer.reduceExportFrom(node, { namedExports: namedExports.map(function (n) {
          return n();
        }) });
    },
    reduceExportFromSpecifier: function reduceExportFromSpecifier(node) {
      return reducer.reduceExportFromSpecifier(node);
    },
    reduceExportLocalSpecifier: function reduceExportLocalSpecifier(node, _ref30) {
      var name = _ref30.name;

      return reducer.reduceExportLocalSpecifier(node, { name: name() });
    },
    reduceExportLocals: function reduceExportLocals(node, _ref31) {
      var namedExports = _ref31.namedExports;

      return reducer.reduceExportLocals(node, { namedExports: namedExports.map(function (n) {
          return n();
        }) });
    },
    reduceExpressionStatement: function reduceExpressionStatement(node, _ref32) {
      var expression = _ref32.expression;

      return reducer.reduceExpressionStatement(node, { expression: expression() });
    },
    reduceForInStatement: function reduceForInStatement(node, _ref33) {
      var left = _ref33.left,
          right = _ref33.right,
          body = _ref33.body;

      return reducer.reduceForInStatement(node, { left: left(), right: right(), body: body() });
    },
    reduceForOfStatement: function reduceForOfStatement(node, _ref34) {
      var left = _ref34.left,
          right = _ref34.right,
          body = _ref34.body;

      return reducer.reduceForOfStatement(node, { left: left(), right: right(), body: body() });
    },
    reduceForStatement: function reduceForStatement(node, _ref35) {
      var init = _ref35.init,
          test = _ref35.test,
          update = _ref35.update,
          body = _ref35.body;

      return reducer.reduceForStatement(node, { init: init == null ? null : init(), test: test == null ? null : test(), update: update == null ? null : update(), body: body() });
    },
    reduceFormalParameters: function reduceFormalParameters(node, _ref36) {
      var items = _ref36.items,
          rest = _ref36.rest;

      return reducer.reduceFormalParameters(node, { items: items.map(function (n) {
          return n();
        }), rest: rest == null ? null : rest() });
    },
    reduceFunctionBody: function reduceFunctionBody(node, _ref37) {
      var directives = _ref37.directives,
          statements = _ref37.statements;

      return reducer.reduceFunctionBody(node, { directives: directives.map(function (n) {
          return n();
        }), statements: statements.map(function (n) {
          return n();
        }) });
    },
    reduceFunctionDeclaration: function reduceFunctionDeclaration(node, _ref38) {
      var name = _ref38.name,
          params = _ref38.params,
          body = _ref38.body;

      return reducer.reduceFunctionDeclaration(node, { name: name(), params: params(), body: body() });
    },
    reduceFunctionExpression: function reduceFunctionExpression(node, _ref39) {
      var name = _ref39.name,
          params = _ref39.params,
          body = _ref39.body;

      return reducer.reduceFunctionExpression(node, { name: name == null ? null : name(), params: params(), body: body() });
    },
    reduceGetter: function reduceGetter(node, _ref40) {
      var name = _ref40.name,
          body = _ref40.body;

      return reducer.reduceGetter(node, { name: name(), body: body() });
    },
    reduceIdentifierExpression: function reduceIdentifierExpression(node) {
      return reducer.reduceIdentifierExpression(node);
    },
    reduceIfStatement: function reduceIfStatement(node, _ref41) {
      var test = _ref41.test,
          consequent = _ref41.consequent,
          alternate = _ref41.alternate;

      return reducer.reduceIfStatement(node, { test: test(), consequent: consequent(), alternate: alternate == null ? null : alternate() });
    },
    reduceImport: function reduceImport(node, _ref42) {
      var defaultBinding = _ref42.defaultBinding,
          namedImports = _ref42.namedImports;

      return reducer.reduceImport(node, { defaultBinding: defaultBinding == null ? null : defaultBinding(), namedImports: namedImports.map(function (n) {
          return n();
        }) });
    },
    reduceImportNamespace: function reduceImportNamespace(node, _ref43) {
      var defaultBinding = _ref43.defaultBinding,
          namespaceBinding = _ref43.namespaceBinding;

      return reducer.reduceImportNamespace(node, { defaultBinding: defaultBinding == null ? null : defaultBinding(), namespaceBinding: namespaceBinding() });
    },
    reduceImportSpecifier: function reduceImportSpecifier(node, _ref44) {
      var binding = _ref44.binding;

      return reducer.reduceImportSpecifier(node, { binding: binding() });
    },
    reduceLabeledStatement: function reduceLabeledStatement(node, _ref45) {
      var body = _ref45.body;

      return reducer.reduceLabeledStatement(node, { body: body() });
    },
    reduceLiteralBooleanExpression: function reduceLiteralBooleanExpression(node) {
      return reducer.reduceLiteralBooleanExpression(node);
    },
    reduceLiteralInfinityExpression: function reduceLiteralInfinityExpression(node) {
      return reducer.reduceLiteralInfinityExpression(node);
    },
    reduceLiteralNullExpression: function reduceLiteralNullExpression(node) {
      return reducer.reduceLiteralNullExpression(node);
    },
    reduceLiteralNumericExpression: function reduceLiteralNumericExpression(node) {
      return reducer.reduceLiteralNumericExpression(node);
    },
    reduceLiteralRegExpExpression: function reduceLiteralRegExpExpression(node) {
      return reducer.reduceLiteralRegExpExpression(node);
    },
    reduceLiteralStringExpression: function reduceLiteralStringExpression(node) {
      return reducer.reduceLiteralStringExpression(node);
    },
    reduceMethod: function reduceMethod(node, _ref46) {
      var name = _ref46.name,
          params = _ref46.params,
          body = _ref46.body;

      return reducer.reduceMethod(node, { name: name(), params: params(), body: body() });
    },
    reduceModule: function reduceModule(node, _ref47) {
      var directives = _ref47.directives,
          items = _ref47.items;

      return reducer.reduceModule(node, { directives: directives.map(function (n) {
          return n();
        }), items: items.map(function (n) {
          return n();
        }) });
    },
    reduceNewExpression: function reduceNewExpression(node, _ref48) {
      var callee = _ref48.callee,
          _arguments = _ref48.arguments;

      return reducer.reduceNewExpression(node, { callee: callee(), _arguments: _arguments.map(function (n) {
          return n();
        }) });
    },
    reduceNewTargetExpression: function reduceNewTargetExpression(node) {
      return reducer.reduceNewTargetExpression(node);
    },
    reduceObjectAssignmentTarget: function reduceObjectAssignmentTarget(node, _ref49) {
      var properties = _ref49.properties;

      return reducer.reduceObjectAssignmentTarget(node, { properties: properties.map(function (n) {
          return n();
        }) });
    },
    reduceObjectBinding: function reduceObjectBinding(node, _ref50) {
      var properties = _ref50.properties;

      return reducer.reduceObjectBinding(node, { properties: properties.map(function (n) {
          return n();
        }) });
    },
    reduceObjectExpression: function reduceObjectExpression(node, _ref51) {
      var properties = _ref51.properties;

      return reducer.reduceObjectExpression(node, { properties: properties.map(function (n) {
          return n();
        }) });
    },
    reduceReturnStatement: function reduceReturnStatement(node, _ref52) {
      var expression = _ref52.expression;

      return reducer.reduceReturnStatement(node, { expression: expression == null ? null : expression() });
    },
    reduceScript: function reduceScript(node, _ref53) {
      var directives = _ref53.directives,
          statements = _ref53.statements;

      return reducer.reduceScript(node, { directives: directives.map(function (n) {
          return n();
        }), statements: statements.map(function (n) {
          return n();
        }) });
    },
    reduceSetter: function reduceSetter(node, _ref54) {
      var name = _ref54.name,
          param = _ref54.param,
          body = _ref54.body;

      return reducer.reduceSetter(node, { name: name(), param: param(), body: body() });
    },
    reduceShorthandProperty: function reduceShorthandProperty(node, _ref55) {
      var name = _ref55.name;

      return reducer.reduceShorthandProperty(node, { name: name() });
    },
    reduceSpreadElement: function reduceSpreadElement(node, _ref56) {
      var expression = _ref56.expression;

      return reducer.reduceSpreadElement(node, { expression: expression() });
    },
    reduceStaticMemberAssignmentTarget: function reduceStaticMemberAssignmentTarget(node, _ref57) {
      var object = _ref57.object;

      return reducer.reduceStaticMemberAssignmentTarget(node, { object: object() });
    },
    reduceStaticMemberExpression: function reduceStaticMemberExpression(node, _ref58) {
      var object = _ref58.object;

      return reducer.reduceStaticMemberExpression(node, { object: object() });
    },
    reduceStaticPropertyName: function reduceStaticPropertyName(node) {
      return reducer.reduceStaticPropertyName(node);
    },
    reduceSuper: function reduceSuper(node) {
      return reducer.reduceSuper(node);
    },
    reduceSwitchCase: function reduceSwitchCase(node, _ref59) {
      var test = _ref59.test,
          consequent = _ref59.consequent;

      return reducer.reduceSwitchCase(node, { test: test(), consequent: consequent.map(function (n) {
          return n();
        }) });
    },
    reduceSwitchDefault: function reduceSwitchDefault(node, _ref60) {
      var consequent = _ref60.consequent;

      return reducer.reduceSwitchDefault(node, { consequent: consequent.map(function (n) {
          return n();
        }) });
    },
    reduceSwitchStatement: function reduceSwitchStatement(node, _ref61) {
      var discriminant = _ref61.discriminant,
          cases = _ref61.cases;

      return reducer.reduceSwitchStatement(node, { discriminant: discriminant(), cases: cases.map(function (n) {
          return n();
        }) });
    },
    reduceSwitchStatementWithDefault: function reduceSwitchStatementWithDefault(node, _ref62) {
      var discriminant = _ref62.discriminant,
          preDefaultCases = _ref62.preDefaultCases,
          defaultCase = _ref62.defaultCase,
          postDefaultCases = _ref62.postDefaultCases;

      return reducer.reduceSwitchStatementWithDefault(node, { discriminant: discriminant(), preDefaultCases: preDefaultCases.map(function (n) {
          return n();
        }), defaultCase: defaultCase(), postDefaultCases: postDefaultCases.map(function (n) {
          return n();
        }) });
    },
    reduceTemplateElement: function reduceTemplateElement(node) {
      return reducer.reduceTemplateElement(node);
    },
    reduceTemplateExpression: function reduceTemplateExpression(node, _ref63) {
      var tag = _ref63.tag,
          elements = _ref63.elements;

      return reducer.reduceTemplateExpression(node, { tag: tag == null ? null : tag(), elements: elements.map(function (n) {
          return n();
        }) });
    },
    reduceThisExpression: function reduceThisExpression(node) {
      return reducer.reduceThisExpression(node);
    },
    reduceThrowStatement: function reduceThrowStatement(node, _ref64) {
      var expression = _ref64.expression;

      return reducer.reduceThrowStatement(node, { expression: expression() });
    },
    reduceTryCatchStatement: function reduceTryCatchStatement(node, _ref65) {
      var body = _ref65.body,
          catchClause = _ref65.catchClause;

      return reducer.reduceTryCatchStatement(node, { body: body(), catchClause: catchClause() });
    },
    reduceTryFinallyStatement: function reduceTryFinallyStatement(node, _ref66) {
      var body = _ref66.body,
          catchClause = _ref66.catchClause,
          finalizer = _ref66.finalizer;

      return reducer.reduceTryFinallyStatement(node, { body: body(), catchClause: catchClause == null ? null : catchClause(), finalizer: finalizer() });
    },
    reduceUnaryExpression: function reduceUnaryExpression(node, _ref67) {
      var operand = _ref67.operand;

      return reducer.reduceUnaryExpression(node, { operand: operand() });
    },
    reduceUpdateExpression: function reduceUpdateExpression(node, _ref68) {
      var operand = _ref68.operand;

      return reducer.reduceUpdateExpression(node, { operand: operand() });
    },
    reduceVariableDeclaration: function reduceVariableDeclaration(node, _ref69) {
      var declarators = _ref69.declarators;

      return reducer.reduceVariableDeclaration(node, { declarators: declarators.map(function (n) {
          return n();
        }) });
    },
    reduceVariableDeclarationStatement: function reduceVariableDeclarationStatement(node, _ref70) {
      var declaration = _ref70.declaration;

      return reducer.reduceVariableDeclarationStatement(node, { declaration: declaration() });
    },
    reduceVariableDeclarator: function reduceVariableDeclarator(node, _ref71) {
      var binding = _ref71.binding,
          init = _ref71.init;

      return reducer.reduceVariableDeclarator(node, { binding: binding(), init: init == null ? null : init() });
    },
    reduceWhileStatement: function reduceWhileStatement(node, _ref72) {
      var test = _ref72.test,
          body = _ref72.body;

      return reducer.reduceWhileStatement(node, { test: test(), body: body() });
    },
    reduceWithStatement: function reduceWithStatement(node, _ref73) {
      var object = _ref73.object,
          body = _ref73.body;

      return reducer.reduceWithStatement(node, { object: object(), body: body() });
    },
    reduceYieldExpression: function reduceYieldExpression(node, _ref74) {
      var expression = _ref74.expression;

      return reducer.reduceYieldExpression(node, { expression: expression == null ? null : expression() });
    },
    reduceYieldGeneratorExpression: function reduceYieldGeneratorExpression(node, _ref75) {
      var expression = _ref75.expression;

      return reducer.reduceYieldGeneratorExpression(node, { expression: expression() });
    }
  };
}